local signals = require 'signals'
local grid = require 'grid'

Bot = class{name = 'Bot', function(self, level, p, d)
	self.pos = assert(p)
	self.dir = d or vector(1,0)
	self.target = self.pos
	self.rot = 0
	self.target_rot = 0
	self.level = level
	self.carries = nil

	signals.clear('move')
	signals.clear('right')
	signals.clear('left')
	signals.clear('pickup')
	signals.clear('drop')
	signals.register('move',  function()
		local target = self.target + self.dir
		if level:walkable(target) then
			self.target = target
		end
	end)
	signals.register('right', function()
		self.dir.x, self.dir.y = -self.dir.y,  self.dir.x
		self.target_rot = self.target_rot + math.pi / 2
	end)
	signals.register('left',  function()
		self.dir.x, self.dir.y =  self.dir.y, -self.dir.x
		self.target_rot = self.target_rot - math.pi / 2
	end)

	signals.register('pickup', function()
		if self.carries then
			--signals.emit('invalid-operation')
			return
		end

		local p = self.target + self.dir
		if level:pickable(p) then
			self.carries = self.carries or level:pickup(p)
			signals.emit('pickup-success')
		else
			signals.emit('invalid-operation')
		end
	end)

	signals.register('drop', function()
		if not self.carries then
			signals.emit('invalid-operation')
			return
		end

		local p = self.target + self.dir
		if level:dropable(p, self.carries) then
			signals.emit('drop-success')
			level:drop(p, self.carries)
			self.carries = nil
		else
			signals.emit('invalid-operation')
		end
	end)

end}

function Bot:draw()
	local x,y = grid.pos(self.pos:unpack())
	love.graphics.setColor(1,1,1)
	love.graphics.draw(Image.nbot, x+20,y+20, self.rot,1,1, 20,20)

	--local x,y = grid.pos((self.target):unpack())
	--love.graphics.rectangle('fill', x+15,y+15, 10,10)

	--local p = self.target + self.dir
	--x,y = math.floor(p.x), math.floor(p.y)
	--x,y = grid.pos(x,y)
	--love.graphics.rectangle('fill', x+25,y+25, 10,10)
end

function Bot:update(dt)
	--self.pos = self.pos + (self.target - self.pos) * .8 * dt
	self.pos = self.target
	--self.rot = self.rot + (self.target_rot - self.rot) * .8 * dt
	self.rot = self.target_rot

	local p = self.target + self.dir
	p.x,p.y = math.floor(p.x), math.floor(p.y)
	if self.level:sense(p) then
		signals.emit('sensor-on')
	else
		signals.emit('sensor-off')
	end

	if self.carries then
		signals.emit('sensor-hand-on')
	else
		signals.emit('sensor-hand-off')
	end
end
