local gui = require 'quickie'
local signals = require 'signals'
require 'circuit'
require 'elements'

local Elements = {'Source', 'Delay', '|', 'NOT', 'AND', 'OR', '|', 'Sensor', 'move', 'right', 'left',  '|', 'SensorHand', 'pickup', 'drop'}

CircuitEditor = class{name = "CircuitEditor", function(self, circuit)
	self.circuit = circuit or Circuit()
	gui.core.disableKeyFocus()
	self.move_mode = {checked = false}
end}

function CircuitEditor:update(dt)
	local x = 5
	for _,what in ipairs(Elements) do
		if what ~= '|' then
			if tail(gui.ImageButton(Image[what], x, HEIGHT-37, 32,32)) then
				self.dragnew = self.dragnew or what
				gui.core.setActive()
			end
			x = x + 32 + 5
		else
			x = x + 10
		end
	end

	gui.Checkbox(self.move_mode, 11,11,12,12)
	if self.move_mode.checked then
		gui.Label('move', 27,8)
	else
		gui.Label('wire', 27,8)
	end

	for elem in pairs(self.circuit.elements) do
		if tail(gui.ImageButton(elem.img, elem.pos.x-16, elem.pos.y-16, 32,32)) then
			if self.move_mode.checked then
				self.move = elem
			else
				self.connection = self.connection or elem
			end
			gui.core.setActive()
		end
	end

	if self.move then
		self.move.pos = vector(love.mouse.getPosition())
	end

	self.circuit:update(dt)
end

function CircuitEditor:mousereleased(x,y, btn)
	if btn == 'r' then
		for elem in pairs(self.circuit.elements) do
			if elem.pos:dist(vector(x,y)) <= 36 then
				self.circuit:remove(elem)
				signals.emit('element-removed')
			end
		end
		return
	end

	if self.dragnew and y < HEIGHT-50 then
		local elem = _G[self.dragnew](x,y)
		self.circuit:add(elem)
		signals.emit('element-placed')
	elseif self.connection then
		for elem in pairs(self.circuit.elements) do
			if elem ~= self.connection and elem.pos:dist(vector(x,y)) <= 36 then
				if self.circuit:getConnection(self.connection, elem) then
					self.circuit:unconnect(self.connection, elem)
					signals.emit('connection-severed')
				elseif not self.circuit:fully_connected(elem) then
					self.circuit:connect(self.connection, elem)
					signals.emit('connection-made')
				else
					signals.emit('connection-invalid')
				end
			end
		end
	elseif self.move then
		if self.move.pos.y >= HEIGHT-50 then
			self.circuit:remove(self.move)
			signals.emit('element-removed')
		else
			signals.emit('element-placed')
		end
	end
	self.dragnew, self.connection, self.move = nil, nil, nil
end

function CircuitEditor:draw()
	self.circuit:draw()

	love.graphics.setLine(3)
	love.graphics.setColor(COLOR'#454545a0')
	love.graphics.rectangle('fill', 0,HEIGHT-42, WIDTH,52)
	love.graphics.line(0,HEIGHT-42,WIDTH,HEIGHT-42)
	love.graphics.setColor(1,1,1)

	if self.dragnew then
		love.graphics.draw(Image[self.dragnew], love.mouse.getX(), love.mouse.getY(), 0,1,1, 16,16)
	elseif self.connection then
		love.graphics.setLine(3)
		love.graphics.setColor(1,1,1,.5)
		love.graphics.line(self.connection.pos.x, self.connection.pos.y, love.mouse.getPosition())
	end
end

function CircuitEditor:keypressed(key)
	if key:sub(2,-1) == 'shift' then
		self.move_mode.checked = not self.move_mode.checked
	end
end

function CircuitEditor:keyreleased(key)
	if key:sub(2,-1) == 'shift' then
		self.move_mode.checked = not self.move_mode.checked
	end
end
