local signals = require 'signals'

Circuit = class{name = 'Circuit', function(self)
	self.elements = {}
	self.connections = {}
	self.charge_frac = 0
	self.charge_frac_update = Timer.Oscillator(3, function(f) self.charge_frac = f end)
	--self.charge_frac_update = Timer.Oscillator(2, function(f) self.charge_frac = (1-math.cos(f*math.pi))/2 end)
end}

function Circuit:update(dt)
	self.charge_frac_update(dt)
end

function Circuit:clear()
	for c in pairs(self.connections) do
		c.consumer.ins[c.producer] = nil
		c.producer.value = nil
	end
end

function Circuit:step()
	for c in pairs(self.connections) do
		c.consumer.ins[c.producer] = c.producer.value
	end
	for elem in pairs(self.elements) do
		elem:step()
	end
	signals.emit('circuit-step')
end

function Circuit:add(elem, ...)
	if not elem then return end
	self.elements[elem] = elem
end

function Circuit:remove(elem)
	local remove = {}

	for c in pairs(self.connections) do
		if c.consumer == elem or c.producer == elem then
			remove[#remove+1] = c
		end
	end

	self.elements[elem] = nil
	for _,c in ipairs(remove) do
		c.consumer.ins[c.producer] = nil
		self.connections[c] = nil
	end
end

function Circuit:nInputs(a)
	local i = 0
	for c in pairs(self.connections) do
		if c.consumer == a then i = i + 1 end
	end
	return i
end

function Circuit:fully_connected(a)
	return self:nInputs(a) >= a.max_inputs
end

function Circuit:getConnection(a,b)
	for c in pairs(self.connections) do
		if c.producer == a and c.consumer == b or c.producer == b and c.consumer == a then
			return c
		end
	end
	return false
end

function Circuit:connect(a,b)
	local link = self:getConnection(a,b)
	if link and link.producer == a then return end
	link = {producer = a, consumer = b}
	self.connections[link] = link
end

function Circuit:unconnect(a,b)
	local link = self:getConnection(a,b)
	if link then
		self.connections[link] = nil
		link.consumer.ins[link.producer] = nil
		self:unconnect(a,b)
	end
end

function Circuit:draw()
	-- draw connection of elements
	for c in pairs(self.connections) do
		if c.producer.value then
			love.graphics.setColor(.6,.8,.2)
		else
			love.graphics.setColor(.5,.5,.5)
		end
		--love.graphics.setLine(6)
		--local d = (c.consumer.pos - c.producer.pos) / 2 + c.producer.pos
		--love.graphics.line(c.producer.pos.x, c.producer.pos.y, d:unpack())
		local d = (c.consumer.pos - c.producer.pos) * self.charge_frac + c.producer.pos
		love.graphics.circle('fill', d.x, d.y, 4,4)
		d = (c.consumer.pos - c.producer.pos) * ((self.charge_frac - .333)%1) + c.producer.pos
		love.graphics.circle('fill', d.x, d.y, 4,4)
		d = (c.consumer.pos - c.producer.pos) * ((self.charge_frac + .333)%1) + c.producer.pos
		love.graphics.circle('fill', d.x, d.y, 4,4)

		love.graphics.setLine(3)
		love.graphics.line(c.producer.pos.x, c.producer.pos.y, c.consumer.pos:unpack())
	end
end
