local signals = require 'signals'

local function get_input(self, i)
	for e in pairs(self.ins) do
		i = i - 1
		if i <= 0 then return e.value end
	end
	return false
end

local function any_input(self)
	for _,v in pairs(self.ins) do
		if v then return true end
	end
	return false
end

local function make_element(name, max_inputs, f)
	local c = class{name = name, function(self, x,y)
		if not vector.isvector(x) then x = vector(x,y) end
		(f or function() end)(self)
		self.pos = x
		self.img = Image[name]
		self.ins = {}
		self.max_inputs = max_inputs or math.huge
	end}
	c.step = _NULL_
	_G[name] = c
end

make_element('Source', 0)
make_element('AND', 2)
make_element('OR', 2)
make_element('NOT', 1)
make_element('Delay', 1)

make_element('Sensor', math.huge, function(self)
	self.value = false;
	signals.register('sensor-on',  function() self.value = true end)
	signals.register('sensor-off', function() self.value = false end)
end)
make_element('move')
make_element('right')
make_element('left')

make_element('SensorHand', math.huge, function(self)
	self.value = false;
	signals.register('sensor-hand-on',  function() self.value = true end)
	signals.register('sensor-hand-off', function() self.value = false end)
end)
make_element('pickup')
make_element('drop')

function Source:step()
	--self.value = not self.value
	self.value = true
end

function AND:step()
	self.value = get_input(self, 1) and get_input(self, 2)
end

function NOT:step()
	self.value = not get_input(self, 1)
end

function OR:step()
	self.value = get_input(self, 1) or get_input(self, 2)
end

function Delay:step()
	self.counter = self.counter or 0
	self.value = false
	if get_input(self, 1) then
		self.counter = self.counter + 1
	end
	if self.counter > 1 then
		self.value = true
		self.counter = 0
	end
end

local function emitter(e, ...)
	if not e then return end
	_G[e].step = function(self)
		self.value = any_input(self)
		if self.value then
			signals.emit(e)
		end
	end
	emitter(...)
end

emitter('move', 'right', 'left', 'pickup', 'drop')
