local grid = require 'grid'
local signals = require 'signals'
require 'bot'

Level = class{name = "Level", function(self, map)
	self.map = map
	self:reset()
end}

function Level:reset()
	local map = self.map
	self.bot = Bot(self, vector(map.bot.pos.x, map.bot.pos.y))
	self.exit = vector(map.exit.x, map.exit.y)

	self.goals = 0
	self.items = {}
	for key,item in pairs(map.items) do
		local i = {img = item.img, pos = vector(item.pos.x, item.pos.y), color = item.color}
		i.key = tostring(i.pos)
		self.items[tostring(i.pos)] = i
		if item.img == 'socket' then
			self.goals = self.goals + 1
		end
	end
end

function Level:update(dt, speed)
	self.bot:update(dt * speed)

	if self.goals <= 0 and self.bot.target:dist(self.exit) < .5 then
		signals.emit('level-complete')
	end
end

local glow = Timer.Oscillator(5,function(f) return (1-math.cos(2*math.pi*f))/2 end)
function Level:draw()
	local a = glow(DT())^5 * .3 + .05
	love.graphics.setColor(.5,.7,1,a)
	love.graphics.setLine(1)
	for i = 0,WIDTH,grid.cell_size do
		love.graphics.line(i,0,i,HEIGHT)
	end
	for k = 0,HEIGHT,grid.cell_size do
		love.graphics.line(0,k,WIDTH,k)
	end

	for _, item in pairs(self.items) do
		love.graphics.setColor(unpack(item.color))
		love.graphics.draw(Image[item.img], grid.pos(item.pos:unpack()))
	end

	love.graphics.setColor(1,1,1)
	love.graphics.draw(Image.exit, grid.pos(self.exit:unpack()))
	self.bot:draw()
end

function Level:walkable(v)
	local ret = true
	ret = ret and v.x > 0 and v.x <= math.ceil(WIDTH/grid.cell_size)
	ret = ret and v.y > 0 and v.y <= math.ceil(HEIGHT/grid.cell_size)
	ret = ret and self.items[tostring(v)] == nil
	return ret
end

function Level:pickable(v)
	local item = self.items[tostring(v)]
	return item and item.img == 'marble'
end

function Level:pickup(v)
	local k = tostring(v)
	local i = self.items[k]
	self.items[k] = nil
	return i
end

function Level:drop(v, item)
	local k = tostring(v)
	if self.items[k] then -- must be a socket
		self.items[k] = {key = k, pos = v:clone(), color = item.color, img = 'filled_socket'}
		self.goals = self.goals - 1
	else
		item.pos = v:clone()
		item.key = k
		self.items[k] = item
	end
end

function Level:sense(v)
	return not self:walkable(v)
end

function Level:dropable(v, item)
	if self:walkable(v) then return true end
	local k = self.items[tostring(v)]
	return k and k.img == 'socket' and k.number == item.number
end
