local gui = require 'quickie'
require 'level'
local grid = require 'grid'

local st = GS.new()
local dummy_map = {bot = {pos = vector(1,1)}, exit = vector(2,2), items = {}}
local itemname = 'marble'
local nextname = {
	marble = 'socket',
	socket = 'wall',
	wall   = 'marble',
}
local itemcolors = {
	{1,0,0}, {0,1,0}, {0,0,1}, {1,1,0}, {1,0,1}, {0,1,1}, {1,1,1}
}

-- gui stuff
local level_file = {text = 'level'}
local clear_input_timer

function st:init()
	local ok, map = pcall(love.filesystem.load, "maps/editor-wip.lua")
	if ok then
		self.level = Level(map())
	else
		self.level = Level(dummy_map)
	end
	st:process_level()
	self.cam = Camera()
	self.cam.zoom = .9

	gui.core.disableKeyFocus()
end

function st:leave()
	self:save_level('editor-wip')
end

function st:process_level()
	self.bot = self.level.bot
	self.items = self.level.items
	self.itemcount = {}
	for _,item in pairs(self.items) do
		self.itemcount[item.img] = (self.itemcount[item.img] or 0) + 1
	end
end

function st:save_level(path)
	local s = Serialize{
		bot   = {pos = self.level.bot.pos},
		exit  = self.level.exit,
		items = self.items,
	}
	love.filesystem.mkdir('maps')
	local f = love.filesystem.newFile(('maps/%s.lua'):format(path))
	f:open('w')
	f:write('return ')
	f:write(s)
	f:close()
end

function st:update()
	love.graphics.setFont(Font.debug[14])
	local x,y = 5,HEIGHT-26
	if gui.Input(level_file, x,y, 200,20) then
		if clear_input_timer then
			Timer.remove(clear_input_timer)
			clear_input_timer = nil
		end
	end; x = x + 200 + 10
	if gui.Button('test', x,y, 70,20) then
		GS.switch(State.game, 'editor-wip', true)
	end;                                x= x + 70 + 5
	if gui.Button('load', x,y, 70,20) then
		local ok, map = pcall(love.filesystem.load, ("maps/%s.lua"):format(level_file.text))
		if ok then
			self.level = Level(map())
			self:process_level()
		else
			level_file.text = '<invalid path>'
			clear_input_timer = Timer.add(1, function() level_file.text = '' end)
		end
	end;                                x = x + 70 + 5
	if gui.Button('save', x,y, 70,20) then
		self:save_level(level_file.text)
	end

	x = WIDTH-70-5
	if gui.Button('menu', x,y, 70,20) then
		GS.switch(State.title)
	end;                                x = x - 70 - 5
	if gui.Button('reset', x,y, 70,20) then
		self.level = Level(dummy_map)
		self:process_level()
	end;

	x,y = WIDTH-75,5
	if gui.Button(itemname, x,y,70,20) then
		itemname = nextname[itemname]
	end;                                x = x - 70 - 5
	gui.Label('Item', x,y-3,70,20, 'right')

	if dragging then
		x,y = grid.cell(self.cam:mousepos())
		dragging.pos.x = math.floor(x+.5)
		dragging.pos.y = math.floor(y+.5)
	end
end

function st:mousepressed(x,y, btn)
	-- remove item
	if btn == 'r' then
		-- XXX: bug when removing marble/socket that is not the newest one: there is no way to create the according socket again
		local p = vector(grid.cell(self.cam:mousepos()))
		local item = self.items[tostring(p)]
		if item then
			self.itemcount[item.img] = self.itemcount[item.img] - 1
		end
		self.items[tostring(p)] = nil
		return
	end

	x,y = grid.cell(self.cam:mousepos())
	if x == self.bot.pos.x and y == self.bot.pos.y then
		dragging = self.bot
		return
	end

	if x == self.level.exit.x and y == self.level.exit.y then
		dragging = {isexit = true, pos = self.level.exit}
		return
	end

	dragging = self.items[tostring(vector(x,y))]
end

function st:mousereleased(x,y, btn)
	if btn == 'r' then return end

	-- drop item
	if dragging then
		if dragging.isexit then
			self.items[tostring(self.level.exit)] = nil
			dragging = nil
			return
		end

		if dragging ~= self.bot then
			self.items[dragging.key] = nil
		end
		dragging.key = tostring(dragging.pos)
		self.items[dragging.key] = dragging
		if dragging == self.bot then
			self.items[dragging.key] = nil
		end
		dragging = nil
		return
	end

	-- place item at cell
	x,y = self.cam:mousepos()
	if x <= 0 or y <= 0 or x >= WIDTH or y >= HEIGHT then return end
	local p = vector(grid.cell(x,y))

	local item = {key = tostring(p), pos = p, img = itemname}
	self.itemcount[item.img] = (self.itemcount[item.img] or 0) + 1
	item.number = self.itemcount[item.img]
	item.color = {1,1,1}
	if itemname == 'marble' or itemname == 'socket' then
		item.color = itemcolors[item.number]
	end
	self.items[tostring(p)] = item
end

function st:draw()
	self.cam:attach()
	love.graphics.setColor(1,1,1)
	love.graphics.draw(Image.bg, 0,0)
	love.graphics.rectangle('line', 0,0, WIDTH, HEIGHT)

	self.level:draw()
	self.cam:detach()
	gui.core.draw()
end

function st:keypressed(...)
	gui.core.keyboard.pressed(...)
end

return st
